<?php

namespace Drupal\tfl\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * {@inheritdoc}
 */
class Login2FactorAdminConfigForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'tfl_config_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['tfl.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('tfl.settings');

    $form['clear'] = [
      '#type' => 'details',
      '#title' => $this->t('Clear all cache'),
      '#open' => TRUE,
    ];
    $form['clear']['cache'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable to clear all cache. (Sometimes configuration changes would not come due to cache.)'),
      '#default_value' => $config->get('cache'),
    ];
    $form['2factor_api'] = [
      '#type' => 'details',
      '#title' => $this->t('API configuration'),
      '#open' => TRUE,
    ];
    $form['2factor_api']['apikey'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#required' => TRUE,
      '#default_value' => $config->get('apikey'),
      '#description' => $this->t('Please enter 2Factor api key.'),
      '#attributes' => [
        'placeholder' => $this->t('Add api key.'),
      ],
    ];

    $form['2factor_status'] = [
      '#type' => 'details',
      '#title' => $this->t('Status'),
      '#open' => TRUE,
    ];
    $form['2factor_status']['status'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable 2Factor authentication on login.'),
      '#default_value' => $config->get('status'),
    ];
    $form['2factor_type'] = [
      '#type' => 'details',
      '#title' => $this->t('OTP TYPE'),
      '#open' => TRUE,
      '#states' => [
        'visible' => [
          'select[name="status"]' => [
            ['checked' => TRUE],
          ],
        ],
      ],
    ];
    $form['2factor_type']['type'] = [
      '#type' => 'select',
      '#title' => $this->t('Type'),
      '#options' => [
        '' => 'Select',
        'SMS' => 'SMS',
        'VOICE' => 'VOICE',
      ],
      '#default_value' => $config->get('type'),
      '#required' => TRUE,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $apikey = trim($form_state->getValue('apikey'));
    $regex = '/[a-zA-Z0-9]/';
    if (!preg_match($regex, $apikey)) {
      $form_state->setErrorByName('apikey', $this->t('Invalid api key. Please enter valid api key.'));
    }

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $apikey = trim($form_state->getValue('apikey'));
    $status = $form_state->getValue('status');
    $type = $form_state->getValue('type');
    $cache = $form_state->getValue('cache');

    $this->config('tfl.settings')->set('apikey', $apikey)->save();
    $this->config('tfl.settings')->set('status', $status)->save();
    $this->config('tfl.settings')->set('type', $type)->save();
    $this->config('tfl.settings')->set('cache', $cache)->save();
    if ($cache == 1) {
      drupal_flush_all_caches();
    }
    parent::submitForm($form, $form_state);
  }

}
